/* vi: expandtab:softtabstop=4:tabstop=4:cindent:cino=>4,g4,h4,+4,c1,C4,(4,u4,^-0
 */
/*

Copyright (c) 2008,2009 SFB/TR-55
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef LIBQLIST_H_
#define LIBQLIST_H_

#include <QList.h>

/** Create a new QList object.
 * Creates a QList object with no type.
 * @return Pointer to the newly created object,
 *      or NULL pointer in case of failure.
 */
QList_t* QListCreateXX(void);

/** Create array-type QList object.
 * @return NULL in case of failure
 */
QList_t* QListCreateArray(void);

/** Create set-type QList object.
 * @return NULL in case of failure.
 */
QList_t* QListCreateSet(void);

/** Add item to list.
 * @param list QList reference. Has to be of array-type
 *      or set-type. 
 * @param item QList reference to be added to @a list.
 *      note that the contents of @a item are not
 *      copied, just the @a item reference is copied.
 * @return @c 0 in case of success, @c -1 in case of failure.
 */
int QListAddItem(QList_t* list, QList_t* item);

/** Create dictionary-type QList.
 * @return NULL in case of failure.
 */
QList_t* QListCreateDict(void);

/** Add item with a key to a dictionary.
 * This routine adds a QList of any type with a
 * key to a dictionary-type QList.
 * 
 * @param list dictionary-type QList reference.
 * @param item QList reference of any type.
 *      Note that the contents of @a item are not
 *      copied.
 * @param key character key.
 *      The key is actually copied, i.e., the 
 *      dictionary does not store the pointer
 *      @a key, but the contents if the 
 *      null-terminated string that @a key 
 *      points to.
 */
int QListAddItemForKey(QList_t* list, QList_t* item, const char* key);

/** Creates an integer-type QList.
 * @param x long integer to be converted
 *      to a machine-independent representation.
 * @return NULL in case of failure.
 */
QList_t* QListCreateInteger(long int x);

/** Creates a boolean-type QList.
 * @param x If @a x is @0, then the boolean type is
 *      interpreted as @a false, all other values
 *      of @a x are interpreted as @a true.
 * @return NULL in case of failure.
 */
QList_t* QListCreateBoolean(int x);


/** Creates a real-type QList.
 * @param x floating-point number to be converted to
 *      a machine independent representation.
 * @return NULL in case of failure.
 */
QList_t* QListCreateReal(double x);

/** Create UTC-type QList.
 * @param t pointer to a null-terminated string that
 *      holds a time in UTC format. The format is
 *      <i>YYYYMMDD</i><b>T</b><i>hhmmss</i>
 *      or
 *      <i>YYYYMMDD</i><b>T</b><i>hhmmss</i><b>Z</b><i>shift</i>
 * @return NULL in case of failure.
 */
QList_t* QListCreateUTC(const char* t);

/** Create data-type QList.
 * @param data pointer to the data to be stored.
 *      The data can be of arbitrary binary type.
 * @param len length of @a data
 * @return NULL in case of failure.
 */
QList_t* QListCreateData(const void* data, size_t len);

/** Create string-type QList.
 * @param s pointer to null-terminated
 *      UTF-8 encoded string.
 * @return NULL in case of failure.
 */
QList_t* QListCreateString(const char* s);

/** Check a complete QList for its constrains.
 * @rreturn @c 0 in case of success, @c -1 in case of failure.
 */
int QListCheck(QList_t* list, int show);

/** Return type of a QList object.
 *
 * Possible values are:
 *
 *       @a QList_PR_NOTHING <br />
 *       @a QList_PR_array <br />
 *       @a QList_PR_set <br />
 *       @a QList_PR_dict <br />
 *       @a QList_PR_integer <br />
 *       @a QList_PR_boolean <br />
 *       @a QList_PR_real <br />
 *       @a QList_PR_utc <br />
 *       @a QList_PR_data <br />
 *       @a QList_PR_string <br />
 *
 * @return type of the QList object.
 */
QList_PR QListType(QList_t* l);

/** Length of a QList.
 * @param l QList of array-type, list-type or dict-type.
 * @return the length, or @c 0 if not applicable.
 */
int QListLength(QList_t* l);

/** Get item in a QList container.
 * @param list QList object, has to be of
 *      array-type, set-type or dict-type.
 * @return item or NULL in case of failure.
 */
QList_t* QListItemAtIndex(QList_t* list, int index);

/** Get dictionary key from QList container.
 * @param list QList object, has to be of dict-type.
 * @return pointer to null-terminated string (the key)
 *      or NULL in case of failure.
 *      Note that the key is <b>not</b> copied.
 */
const char* QListKeyAtIndex(QList_t* list, int index);

/** Get an item for a given key.
 * @param list QList object, has to be of dict-type.
 * @param key  pointer to null-terminated string, the key.
 * @return pointer to item object,
 *      or NULL in case of failure.
 *      Note that the key is <b>not</b> copied.
 */
QList_t* QListItemForKey(QList_t* list, const char* key);

/** Convert QList object to long int.
 * @param list QList object, has to be of integer-type.
 * @param err error marker. If <code>err == NULL</code>,
 *      it is used to return an error code.
 *      <code>*err == 0</code> means the conversion was OK.
 * @return a long integer representing the OList integer-type
 *      object.
 */
long int QListAsLong(QList_t* list, int* err);

/** Convert QList object to boolean.
 * @param list QList object, has to be of boolean-type.
 * @param err error marker. See @a QListAsLong.
 * @return @c 0 if @a list represents @a false,
 *      or a non-zero integer, if @a list represents @a true.
 */
int QListAsBoolean(QList_t* list, int* err);

/** Convert QList object to double.
 * @param list QList object, has to be of real-type.
 * @param err error marker, Seen @a QListAsLong.
 * @return a double value.
 */
double QListAsDouble(QList_t* list, int* err);

/** Convert QList object to time.
 * @param list QList object, has to be of utc-type.
 * @param err error marker. Seen @a QListAsLong.
 * @return a @c time_t value.
 *      Note that most systems internally define
 *      @c time_t as @c long, so it inherits all the
 *      problems related to that representation,
 *      like the year-2038-problem.
 */
time_t QListAsTime(QList_t* list, int* err);

/** Get the length of a QList.
 * @param list QList object, has to be of data-type.
 * @param err error marker. See @a QListAsLong.
 * @return length of the data.
 */
int QListDataLength(QList_t* list, int* err);

/** Get raw QList data.
 * @param list QList object, has to be of data-type.
 * @param err error marker. See @a QListAsLong.
 * @return pointer to the raw data of the QList object,
 *      or @c NULL in case of failure.
 *      Note that the data is <b>not</b> copied.
 */
const unsigned char* QListDataBuffer(QList_t* list, int* err);

/** Get QList string.
 * @param list QList object, has to be of string-type.
 * @param err error marker. See @a QListAsLong.
 * @return pointer to the NULL-terminated string.
 *      Note that the string is <b>not</b> copied.
 */
const char* QListAsString(QList_t* list, int* err);

/** Dump a human-readable version of a QList object.
 * @param fd file descriptor where the representation
 *      is dumped upon.
 * @param list QList object of any type.
 * @param err error marker. See @a QListAsLong.
 * @param indent initial indentation level.
 */
void QListDump(int fd, QList_t* list, int* err, int indent);

#endif

